unit pgunit;

{
 procedures for polar graph
}

interface

uses windows,graphics,classes,sysutils;  //windows must be mentioned first

procedure paintpolargraphmap;
procedure createPGmap;
procedure destroyPGmap;
procedure drawPGmap;
procedure simreport(a,v : single); //a : angle; v : velocity
procedure showpgmap;
procedure pgTimeErase;

implementation

uses unit1;

type PDW = ^dword;

const polargridcolor1 = $ffc0c0;     //rr-gg-bb  distances 2,4,6...
      polargridcolor2 = $ff8080;     //distances 1,3,7..
      polargridbackground = $fffff0;
      polargraphcolor = $0000ff;

var PGmap : TBitmap;
    pg0 : dword;
    pglinestep : dword;
//
    eraseline : word;

procedure simreport (a,v : single);
//add point at angle a , velocity v
var x,y : smallInt;
    wx,wy : word;   
    r : single;
    p : dword;
    dw :dword;
begin
 r := abs(40*v);
 x := round(r*sin(a));
 y := round(r*cos(a));
 if (abs(x) < 220) and (abs(y) < 220) then
  begin
   wx := 220 + x;
   wy := 220 - y;
   p := pg0-pglinestep*wy+(wx shl 2);
   dw := PDW(p)^;
   dw := dw and $c0000000;
   dw := dw or $1f000000 or polargraphcolor;  //1f is erasetime : 32 x
   PDW(p)^ := dw;
  end;
end;

function RBswap(c : dword) : dword;
//swap red - blue colors
begin
 result := (c and $0000ff00);
 result := result or ((c shr 16) and $ff);
 result := result or ((c and $ff) shl 16);
end;

procedure showpgmap;
begin
 form1.PaintBox2.Canvas.Draw(0,0,pgmap);
end;

procedure createPGmap;
//polar graph map
begin
 PGmap := TBitmap.create;
 with PGmap do
  begin
   pixelformat := pf32bit;
   width := form1.paintbox2.Width;
   height := form1.paintbox2.height;
   pg0 := dword(PGmap.scanline[0]);
   pglinestep := pg0 - dword(pgmap.scanline[1]);
  end;
end;

procedure destroyPGmap;
begin
 pgmap.Free;
end;

procedure drawPGmap;
begin
 form1.PaintBox2.Canvas.Draw(0,0,PGmap);
 eraseline := 0;
end;

procedure paintpolargraphmap;
//paint empty map
var x,y : smallInt;
    wy : word;
    cc,i : byte;
    p : PDW;
    dw : dword;
begin
 with pgmap do with canvas do
  begin
   brush.color := RBswap(polargridbackground);
   brush.style := bsSolid;
   fillrect(rect(0,0,width,height));
   pen.width := 1;
   brush.style := bsClear;
   for i := 1 to 10 do
    begin
     x := 20*i;
     if (i and 1) = 1 then pen.Color := RBswap(polargridcolor1)
      else pen.color := RBswap(polargridcolor2);
     ellipse(220-x,220-x,220+x,220+x);
    end;
   for i := 0 to 11 do
    begin
     x := round(200*cos(i*0.2618)); //pi*15/180
     y := round(200*sin(i*0.2618));
     moveto(220-x,220+y);
     lineto(220+x,220-y);
    end;
   font.name := 'arial';
   font.height := 14;
   font.style := [fsBold];
   font.color := RBswap(polargridcolor2);
   brush.style := bsClear;
   textout(400,205,'m/sec');
   for i := 1 to 5 do textout(220+40*i,220,inttostr(i));
  end;
//----color code setting
// 00,01 : background    10 : grid   11 : text
 for wy := 0 to pgmap.Height-1 do
  begin
   p := PDW(pg0-wy*pglinestep);
   for x := 0 to pgmap.Width-1 do
    begin
     dw := p^;
     case dw of
      polargridbackground : cc := 1;
      polargridcolor1     : cc := 2;
      polargridcolor2     : cc := 3;
      else cc := 0;
     end;
     dw := dw or (cc shl 30);
     p^ := dw;
     inc(p);
    end;
  end;
end;

procedure pgTimeErase;
//scan 1 line of polar graph to bleach old points
var  p : PDW;
     dw : dword;
     x : word;
     cc,tt : byte;
begin
 p := PDW(pg0-eraseline*pglinestep);
 for x := 0 to pgmap.Width-1 do
  begin
   dw := p^;
   tt := (dw shr 24) and $3f;
   if tt > 0 then
    begin
     dec(tt);
     dw := (dw and $c0ffffff) or (tt shl 24);
     if tt = 0 then
      begin
       cc := dw shr 30;
       dw := dw and $c0000000;
       case cc of
        0,1 : dw := dw or polargridbackground;
        2   : dw := dw or polargridcolor1;
        3   : dw := dw or polargridcolor2;
       end;//case
      end else dw := dw + $00080800;//bleach;
     p^:= dw;
    end;//tt>0
    inc(p);
  end;//for
 inc(eraseline);
 if eraseline = pgmap.height then eraseline := 0;
end;

end.
